package edu.berkeley.cs160.teammoomoo.nutritrak160;

import java.io.FileNotFoundException;
import java.sql.Timestamp;
import java.sql.Date;
import java.util.*;

import android.content.SharedPreferences.Editor;
import android.content.Context;
import android.content.ContentValues;
import android.content.SharedPreferences;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.database.Cursor;
import android.util.Log;
import android.util.DateFormat;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewInflate;
import android.view.Window;
import android.view.Menu;
import android.view.Menu.Item;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ScrollView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.LinearLayout.LayoutParams;

/**
 * Simple helper class for the foods database
 * @author Bruno, Ben
 */

public class NutriTrakDdAdapter {
	public static final String FAVORITES_CATEGORY_NAME = "favorites";
	
    //shared attribute names
    public static final String KEY_ROWID="_id";
    public static final String KEY_STATS_ID="stats_id";
    public static final String KEY_CATEGORY_ID="category_id";
    public static final String KEY_NAME="name";
    public static final String KEY_TIMESTAMP="tstamp";
    
    //exercise table
    public static final String TABLE_EXERCISES="exercises";
    public static final String KEY_DURATION="duration";
    
    //food table
    public static final String TABLE_FOOD="foods";
    public static final String KEY_FAVORITISM="favoritism";
    public static final String KEY_UNITS="units";  // deprecated
    
    //stats table
    public static final String TABLE_STATS="stats";
    public static final String[] STATS_TABLE_NUTRI_KEYS = {
    	"calories", "fat", "sugar", "carbohydrates", "protein",
    	"vitaminA", "vitaminC"
    };
    // Unlikely we'll have to refer to specific nutrients, so cram into String array
    // If need specific references, add as instance var later
    //TODO: add the remainder of food properties that we will use in prototype
    
    //exercise
    public static final String[] EXERCISE_KEYS = {
    	"Running", "Swimming", "Bicycling", "Elliptical", "Situps"
    };
    
    //meals table
    public static final String TABLE_MEALS="meals";
    
    //consumed table
    public static final String TABLE_CONSUMED="consumed";
    public static final String KEY_FOOD_ID="food_id";
    public static final String KEY_MEAL_ID="meal_id";
    public static final String KEY_QUANTITY="quantity";
    
    //unit definitions
    // deprecated
    public static String UNIT_SERVING = "servings";
    public static String UNIT_OZ = "oz";
    public static String UNIT_FLOS = "floz";
    
    public static final String TABLE_CATEGORIES="categories";
    
    private static final String EXERCISE_TABLE_CREATE = 
    "create table exercises (" +
    	"_id integer primary key autoincrement, "+
    	"name text not null, " +
    	"duration int not null, " +
    	"tstamp timestamp not null);";
    
    private static final String CATEGORY_TABLE_CREATE =
	"create table categories (" +
		"_id integer primary key autoincrement, " +
		"name text not null, " +
		"category_id integer);"; //parent category id
    
    private static final String FOOD_TABLE_CREATE = 
	"create table foods (" +
		"_id integer primary key autoincrement, " + 
		"name text not null, " +
		"units text not null," + // TODO: units field has been deprecated
		"stats_id integer not null, " +
		"favoritism integer not null, " + 
		"category_id integer not null," + 
		"constraint category_id_fk foreign key(category_id) references categories(_id) on delete cascade," +
		"constraint stats_id_fk foreign key(stats_id) references stats(_id) on delete cascade);";
    
    private static String STAT_TABLE_CREATE = null;
    // Dynamically generated
    // See generateStatTableCreateStatement()
    // Note: all stats are per ounce
    
    private static final String MEAL_TABLE_CREATE = 
	"create table meals (" +
		"_id integer primary key autoincrement, " +
		"tstamp timestamp not null);";
    // Used to refer to stats_id, but instead now dynamically recompute stats
    
    private static final String CONSUMED_TABLE_CREATE =
	"create table consumed ( " + 
		"_id integer primary key autoincrement, " +
		"food_id integer not null, " +
		"meal_id integer not null, " +
		"quantity integer not null," + // this is in ounces
		"constraint food_id_fk foreign key(food_id) references foods(_id) on delete cascade," + 
		"constraint meal_id_fk foreign key(meal_id) references meals(_id) on delete cascade);";
    
    private static final String DATABASE_NAME = "nutritrak_data";
    private static final int DATABASE_VERSION = 1;
    
    private SQLiteDatabase mDb;
    private final Context mCtx;
    private Vector<Integer> correspondingIDs = new Vector<Integer>();  // see getCorrespondingIDs();
    public Hashtable<String,NutrientEntry> nutrientEntries = new Hashtable<String,NutrientEntry>();
    public Hashtable<String,ExerciseData> exerciseEntries = new Hashtable<String,ExerciseData>();
    
    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public NutriTrakDdAdapter(Context ctx) {
        this.mCtx = ctx;
    }
    
    /**
     * Open the notes database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public NutriTrakDdAdapter createAndOpen() throws SQLException {
    	this.generateStatTableCreateStatement();
        if (this.open()) {
        	Log.d("NutriTrakDdAdapter", "Only opening");
            // Only goes past here if db already exists
            // Delete database so that we can always start with clean slate
            // Easier for debugging-- remove later 
            // Basically, this is only for demonstration purposes, will not keep data!!!!
            // If want to retain data, remove initialize call and comment out below kill db code
//            Log.d("NutriTrackDdAdapter", "Deleting old db");
//            this.close();
//            Utils.assertTrue(mCtx.deleteDatabase(DATABASE_NAME));
        	return this;
        }
    	int i = 1;
        try {
        	Log.d("NutriTrackDdAdapter", "Creating database");
            mDb = mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0, null);
            mDb.execSQL(CATEGORY_TABLE_CREATE); i++;
            mDb.execSQL(STAT_TABLE_CREATE); i++;
            mDb.execSQL(FOOD_TABLE_CREATE); i++;
            mDb.execSQL(MEAL_TABLE_CREATE); i++;
            mDb.execSQL(CONSUMED_TABLE_CREATE); i++;
            mDb.execSQL(EXERCISE_TABLE_CREATE); i++;
            Log.d("NutriTrackDdAdapter", "Successfully created tables");
            // Demonstration data
            this.initialize();
        } catch (FileNotFoundException e1) {
            throw new SQLException("Could not create database");
        } catch (SQLException e2) {
        	throw new SQLException("Invalid SQL statement executed as string: Statement " + i + "\n" +
        							"Error: " + e2.toString());
        }
        return this;
    }
    
    /*
     * Really should redo these parts of the code at some point, it's kind of hacked right now 
     * just to make merging work with the fact that there is only 1 db and all the screens use it
     */
    public boolean open() {
//   	 Need to do this every time because deleted for each instance, could make static later
    	this.populateConversionTable();
    	this.populateExerciseTable();
    	try {
    		this.mDb = mCtx.openDatabase(DATABASE_NAME, null);
    	} catch (Exception e) {
    		return false;
    	}
    	return true;
    }

    public void close() {
        mDb.close();
    }
    
    /**
     * This contains demonstration data.  Most of it is not real, only for
     * purposes of demonstrating the interaction, as filling a db is not
     * the purpose of this course.  Also, need to create some previous history
     * as if user was already using this device, otherwise our graph will be
     * blank, etc
     *
     */
    private void initialize() {
    	this.populateDatabase();
//    	this.fakeHistory();
    }
    
    /**
     * Just make up some default history for demonstration purposes
     *
     */
    private void fakeHistory() {
    	int cokeID = this.fromFoodNameToID("Coke");
    	int bkcb = this.fromFoodNameToID("Burger King Bacon Cheeseburger");
    	LinkedList<Integer> foods = new LinkedList<Integer>();
    	LinkedList<Integer> quantities = new LinkedList<Integer>();
    	foods.add(cokeID);
    	foods.add(bkcb);
    	quantities.add(16);  // 16 ounces = 1 drink 
    	quantities.add(5);  // 5 ounces = 1 burger
    	this.createMeal(foods, quantities);
    }
    
    /**
     * Method to call in db.open() to dynamically generate sql create stat table statement  
     * based on the types of nutrients currently supported
     */
    private void generateStatTableCreateStatement () {
    	StringBuilder statement = new StringBuilder();
    	
    	statement.append(
    	"create table stats (" +
    		"_id integer primary key autoincrement");
    	
    	for (String nutrient : STATS_TABLE_NUTRI_KEYS) {
    		statement.append(", ");
    		statement.append(nutrient + " integer not null");
    	}
    	statement.append(");");
    	
    	Log.d("NutriTrackDdAdapter", "generateStatTableCreateStatement:"+statement);
    	NutriTrakDdAdapter.STAT_TABLE_CREATE = statement.toString();
    }
    
    /**
     * Fills the database with a set of default foods and nutrients
     */
    public void populateDatabase() throws SQLException {
    	populateDatabaseSampleData();    	
        Log.d("NutriTrackDdAdapter", "Populated Database successfully");
    }
    
    /**
     * Fills the database with some sample data
     * Proof of concept, not meant to be rigorous
     * Later switch in a more complete set of data
     */
    private void populateDatabaseSampleData() {
    	/*
    	 * Just a small sample to get the general idea
    	 * Two toplevel categories: pies and drinks
    	 * Pie is divided into fruit pie or cream pie (arbitrary)
    	 * Drink is divided into fruit juice or soda (arbitrary)
    	 * 
    	 * Top level categories will have a categoryID of 0
    	 * 
    	 * There will be two fruit pies-- banana and apple.
    	 * There will be 1 boston cream pie in the other cat.
    	 * 
    	 * There will be 1 type of fruit juice-- orange.
    	 * Soda will have coke.
    	 */
    	
    	// Top level Pie category 
    	ContentValues cv = new ContentValues();
    	cv.put(KEY_NAME, "pie");
    	cv.put(KEY_CATEGORY_ID, 0);
    	long pieRowID = this.mDb.insert("categories", null, cv);
    	Utils.assertTrue(pieRowID != -1);

    	// Second level Fruit Pie category
    	cv.clear();
    	cv.put(KEY_NAME, "fruit pie");
    	cv.put(KEY_CATEGORY_ID, pieRowID);
    	long fruitPieID = this.mDb.insert("categories", null, cv);
    	Utils.assertTrue(fruitPieID != -1);
    	
    	// Second level Cream Pie Category
    	cv.clear();
    	cv.put(KEY_NAME, "cream pie");
    	cv.put(KEY_CATEGORY_ID, pieRowID);
    	long creamPieID = this.mDb.insert("categories", null, cv);
    	Utils.assertTrue(creamPieID != -1);
    	
    	/*
    	 * A bit of controversy over storage, eg prefer daily values or in grams???
    	 * Some not given in one but the other...
    	 * Bruno's Policy: Prefer Grams when possible, else use DVs
    	 */
    	
    	/*
    	 * Unit Table:
    	 * Vitamins in DVs
    	 * Calories is its own unit
    	 * Everything else (eg fat) is in grams
    	 */
    	
    	/*
    	 * Note:
    	 * 1 Serving = 1 Ounce here
    	 */

    	// Store a blueberry pie and its nutritional information
    	// http://www.nutritiondata.com/facts-C00001-01c21Jn.html
    	cv.clear();
    	cv.put("calories", 65);  
    	cv.put("fat", 3);  
    	cv.put("sugar", 3); 
    	cv.put("carbohydrates", 10);  
    	cv.put("protein", 1);  
    	cv.put("vitaminA", 1);  
    	cv.put("vitaminC", 1);  
    	long blueBerryPieStatID = this.mDb.insert("stats", null, cv);  
    	Utils.assertTrue(blueBerryPieStatID != -1);
    	cv.clear();
    	cv.put(KEY_NAME, "blueberry pie");
    	cv.put(KEY_FAVORITISM, 0);
    	cv.put(KEY_UNITS, UNIT_SERVING);
    	cv.put(KEY_STATS_ID, blueBerryPieStatID);
    	cv.put(KEY_CATEGORY_ID, fruitPieID);
    	long blueBerryPieID = this.mDb.insert("foods", null, cv);
    	Utils.assertTrue(blueBerryPieID != -1);
    	
    	// Store an apple pie and its nutritional information
    	// http://www.nutritiondata.com/facts-C00001-01c21Jj.html
    	cv.clear();
    	cv.put("calories", 66);  
    	cv.put("fat", 3);  
    	cv.put("sugar", 4); 
    	cv.put("carbohydrates", 10);  
    	cv.put("protein", 1);  
    	cv.put("vitaminA", 1);  
    	cv.put("vitaminC", 1);  
    	long applePieStatID = this.mDb.insert("stats", null, cv);  
    	Utils.assertTrue(applePieStatID != -1);
    	cv.clear();
    	cv.put(KEY_NAME, "apple pie");
    	cv.put(KEY_FAVORITISM, 0);
    	cv.put(KEY_UNITS, UNIT_SERVING);
    	cv.put(KEY_STATS_ID, applePieStatID);
    	cv.put(KEY_CATEGORY_ID, fruitPieID);
    	long applePieID = this.mDb.insert("foods", null, cv);
    	Utils.assertTrue(applePieID != -1);
    	
    	// Store a boston cream pie and its nutritional information
    	// http://www.nutritiondata.com/facts-C00001-01c21Gr.html
    	cv.clear();
    	cv.put("calories", 71);  
    	cv.put("fat", 2);  
    	cv.put("sugar", 10); 
    	cv.put("carbohydrates", 12);  
    	cv.put("protein", 1);  
    	cv.put("vitaminA", 0);  
    	cv.put("vitaminC", 0);  
    	long bostonCreamPieStatID = this.mDb.insert("stats", null, cv);
    	Utils.assertTrue(bostonCreamPieStatID != -1);
    	cv.clear();
    	cv.put(KEY_NAME, "boston cream pie");
    	cv.put(KEY_FAVORITISM, 0);
    	cv.put(KEY_UNITS, UNIT_SERVING);
    	cv.put(KEY_STATS_ID, bostonCreamPieStatID);
    	cv.put(KEY_CATEGORY_ID, creamPieID);
    	long bostonCreamPieID = this.mDb.insert("foods", null, cv);
    	Utils.assertTrue(bostonCreamPieID != -1);
    	
    	// Top level Drink category
    	cv.clear();
    	cv.put(KEY_NAME, "drink");
    	cv.put(KEY_CATEGORY_ID, 0);
    	long drinkRowID = this.mDb.insert("categories", null, cv);
    	Utils.assertTrue(drinkRowID != -1);
    	
    	// Second level Fruit Juice category
    	cv.clear();
    	cv.put(KEY_NAME, "fruit juice");
    	cv.put(KEY_CATEGORY_ID, drinkRowID);
    	long fruitJuiceID = this.mDb.insert("categories", null, cv);
    	Utils.assertTrue(fruitJuiceID != -1);
    	
    	// Second level Soda Category
    	cv.clear();
    	cv.put(KEY_NAME, "soda");
    	cv.put(KEY_CATEGORY_ID, drinkRowID);
    	long sodaID = this.mDb.insert("categories", null, cv);
    	Utils.assertTrue(sodaID != -1);
    	
    	// Store an orange juice and its nutritional info
    	// http://www.nutritiondata.com/facts-C00001-01c21y4.html
    	cv.clear();
    	cv.put("calories", 15);  
    	cv.put("fat", 0);  
    	cv.put("sugar", 3); 
    	cv.put("carbohydrates", 4);  
    	cv.put("protein", 0);  
    	cv.put("vitaminA", 0);  
    	cv.put("vitaminC", 7);  
    	long orangeJuiceStatID = this.mDb.insert("stats", null, cv);
    	Utils.assertTrue(orangeJuiceStatID != -1);
    	cv.clear();
    	cv.put(KEY_NAME, "orange juice");
    	cv.put(KEY_FAVORITISM, 0);
    	cv.put(KEY_UNITS, UNIT_SERVING);
    	cv.put(KEY_STATS_ID, orangeJuiceStatID);
    	cv.put(KEY_CATEGORY_ID, fruitJuiceID);
    	long orangeJuiceID = this.mDb.insert("foods", null, cv);
    	Utils.assertTrue(orangeJuiceID != -1);
    	
    	// Store a coke and its nutritional info
    	// http://www.nutritiondata.com/facts-C00001-01c21la.html
    	cv.clear();
    	cv.put("calories", 12);  
    	cv.put("fat", 0);  
    	cv.put("sugar", 3); 
    	cv.put("carbohydrates", 3);  
    	cv.put("protein", 0);  
    	cv.put("vitaminA", 0);  
    	cv.put("vitaminC", 0);  
    	long cokeStatsID = this.mDb.insert("stats", null, cv);
    	Utils.assertTrue(cokeStatsID != -1);
    	cv.clear();
    	cv.put(KEY_NAME, "Coke");
    	cv.put(KEY_FAVORITISM, 0);
    	cv.put(KEY_UNITS, UNIT_SERVING);
    	cv.put(KEY_STATS_ID, cokeStatsID);
    	cv.put(KEY_CATEGORY_ID, sodaID);
    	long cokeID = this.mDb.insert("foods", null, cv);
    	Utils.assertTrue(cokeID != -1);
    	
        /**
         * Populates the database with a bunch of real categories and foods
         * but fictitious nutritional data.
         */
    	// More top level categories
    	long fastFoodTopLevelID = this.makeUpCategory("fast food", 0);
    	long pastaTopLevelID = this.makeUpCategory("pasta", 0);
    	long snacksTopLevelID = this.makeUpCategory("snacks", 0);
    	long soupsTopLevelID = this.makeUpCategory("soups", 0);
    	long poultryTopLevelID = this.makeUpCategory("poultry", 0);
    	long beefTopLevelID = this.makeUpCategory("beef", 0);
    	long ethnicFoodsTopLevelID = this.makeUpCategory("ethnic foods", 0);

    	// Juices
    	this.makeUpFood("apple juice", fruitJuiceID);
    	this.makeUpFood("lychee juice", fruitJuiceID);
    	this.makeUpFood("grape juice", fruitJuiceID);
    	this.makeUpFood("grapefruit juice", fruitJuiceID);
    	this.makeUpFood("pineapple juice", fruitJuiceID);
    	this.makeUpFood("mango juice", fruitJuiceID);

    	// Sodas
    	this.makeUpFood("Pepsi", sodaID);
    	this.makeUpFood("7 up", sodaID);
    	this.makeUpFood("ginger ale", sodaID);
    	this.makeUpFood("Sprite", sodaID);
    	this.makeUpFood("Dr Pepper", sodaID);
    	this.makeUpFood("Diet Pepsi", sodaID);
    	this.makeUpFood("Diet Coke", sodaID);
    	this.makeUpFood("grape soda", sodaID);
    	this.makeUpFood("Jones Soda Sour Apple", sodaID);
    	this.makeUpFood("Jones Soda Cream", sodaID);
    	
    	// Pies
    	this.makeUpFood("raspberry pie", fruitPieID);
    	this.makeUpFood("blackberry pie pie", fruitPieID);
    	this.makeUpFood("cherry pie", fruitPieID);
    	
    	long miscPieID = this.makeUpCategory("miscellaneous pies", pieRowID);
    	this.makeUpFood("pumpkin pie", miscPieID);
    	this.makeUpFood("pecan pie", miscPieID);
    	this.makeUpFood("egg custard pie", miscPieID);

    	long meatPieID = this.makeUpCategory("meat pies", pieRowID);
    	this.makeUpFood("lamb meat pie", meatPieID);
    	this.makeUpFood("cow meat pie", meatPieID);
    	
    	// Fast Foods
    	long mcID = this.makeUpCategory("Mc Donalds", fastFoodTopLevelID);
    	long wendyID = this.makeUpCategory("Wendys", fastFoodTopLevelID);
    	long burgKingID = this.makeUpCategory("Burger King", fastFoodTopLevelID);
    	long inAndOutID = this.makeUpCategory("In 'n Out", fastFoodTopLevelID);
    	long tacoBellID = this.makeUpCategory("Taco Bell", fastFoodTopLevelID);
    	
    	this.makeUpFood("Mc Donalds hamburger", mcID);
    	this.makeUpFood("Mc Donalds cheeseburger", mcID);
    	this.makeUpFood("Mc Donalds fries", mcID);
    	this.makeUpFood("Mc Donalds chicken nuggets", mcID);
    	this.makeUpFood("Mc Donalds crispy chicken sandwich", mcID);
    	
    	this.makeUpFood("Wendys hamburger", wendyID);
    	this.makeUpFood("Wendys cheeseburger", wendyID);
    	this.makeUpFood("Wendys fries", wendyID);

    	// Ok this one is real
    	// http://www.nutritiondata.com/facts-C00001-01c21l1.html
    	cv.clear();
    	cv.put("calories", 79);  
    	cv.put("fat", 4);  
    	cv.put("sugar", 1); 
    	cv.put("carbohydrates", 6);  
    	cv.put("protein", 4);  
    	cv.put("vitaminA", 1);  
    	cv.put("vitaminC", 0);  
    	long baconCheeseStatsID = this.mDb.insert("stats", null, cv);
    	Utils.assertTrue(baconCheeseStatsID != -1);
    	cv.clear();
    	cv.put(KEY_NAME, "Burger King Bacon Cheeseburger");
    	cv.put(KEY_FAVORITISM, 0);
    	cv.put(KEY_UNITS, UNIT_SERVING);
    	cv.put(KEY_STATS_ID, baconCheeseStatsID);
    	cv.put(KEY_CATEGORY_ID, burgKingID);
    	long baconCheeseID = this.mDb.insert("foods", null, cv);
    	Utils.assertTrue(baconCheeseID != -1);
    	
    	this.makeUpFood("In 'n Out hamburger", inAndOutID);
    	this.makeUpFood("In 'n Out cheeseburger", inAndOutID);
    	this.makeUpFood("In 'n Out fries", inAndOutID);

    	this.makeUpFood("Taco Bell Breakfast Burrito", tacoBellID);
    	this.makeUpFood("Taco Bell Chili Cheese Burrito", tacoBellID);

    	// Pastas
    	long macaroniID = this.makeUpCategory("macaroni", pastaTopLevelID);
    	long cookedMacaroniID = this.makeUpCategory("cooked", macaroniID);this.makeUpCategory("uncooked", macaroniID); 
    	long dryCookedMacaroniID = this.makeUpCategory("dry", cookedMacaroniID);
    	long proteinFortifieddryCookedMacaroniID = this.makeUpCategory("protein-fortified", dryCookedMacaroniID);
    	long wholeWeatproteinFortifieddryCookedMacaroniID = this.makeUpFood("whole wheat protien fortified dry cooked macaroni", proteinFortifieddryCookedMacaroniID);
    	
    	this.makeUpCategory("chinese noodles", pastaTopLevelID);
    	this.makeUpCategory("quinoa", pastaTopLevelID);
    	this.makeUpCategory("rice glutinous", pastaTopLevelID);
    	this.makeUpCategory("spinach", pastaTopLevelID);
    	this.makeUpCategory("japanese soba", pastaTopLevelID);
    	
    	// Snacks
    	this.makeUpFood("granola bar", snacksTopLevelID);
    	this.makeUpFood("pretzel", snacksTopLevelID);
    	this.makeUpFood("chex mix", snacksTopLevelID);
    	this.makeUpFood("popcorn", snacksTopLevelID);
    	this.makeUpFood("almond oatmeal", snacksTopLevelID);
    	this.makeUpFood("apple turnover", snacksTopLevelID);

    	// Poultry
    	this.makeUpFood("raw chicken", poultryTopLevelID);
    	
    	// Beef
    	this.makeUpFood("raw beef", beefTopLevelID);
    	
    	// Soups
    	this.makeUpFood("gazpacho", soupsTopLevelID);
    	
    	this.makeUpFood("mexican salsa", ethnicFoodsTopLevelID);
    }
    
    private long makeUpFood(String name, long parentCategoryID) {
    	ContentValues cv = new ContentValues();
    	Random randGen = new Random();
    	for (String nutri : STATS_TABLE_NUTRI_KEYS) {
    		cv.put(nutri, randGen.nextInt(10));
    	}
    	long statsID = this.mDb.insert("stats", null, cv);
    	Utils.assertTrue(statsID != 1);
    	cv.clear();
    	cv.put(KEY_NAME, name);
    	cv.put(KEY_UNITS, UNIT_SERVING);
    	cv.put(KEY_FAVORITISM, 0);
    	cv.put(KEY_STATS_ID, statsID);
    	cv.put(KEY_CATEGORY_ID, parentCategoryID);
    	long foodID = this.mDb.insert("foods", null, cv);
    	Utils.assertTrue(foodID != -1);
    	return foodID;
    }
    
    private long makeUpCategory(String name, long parentCategoryID) {
    	ContentValues cv = new ContentValues();
    	cv.put(KEY_NAME, name);
    	cv.put(KEY_CATEGORY_ID, parentCategoryID);
    	long catID = this.mDb.insert("categories", null, cv);
    	Utils.assertTrue(catID != -1);
    	return catID;
    }
    
    public enum NutriUnit {DAILY_VALUE, GRAMS, SELF};
    /**
     * A class representing a nutrient entry in a hashtable whose keys are
     * the names of the nutrients.  This class will allow us to keep track
     * of nutrient unit information and conversion factors etc
     */
    public static class NutrientEntry {
    	/** 
    	 * An enum value indicating whether the value stored in the stats table 
    	 * is of grams or DV or inherent unit (eg for calories) */
    	NutriUnit unit;
    	/** 
    	 * If the unit is grams, how many grams equals 1 DV.
    	 * This field is -1 if the nutrient is unnecessary (eg sugar) or if
    	 * the unit is not grams. */
    	int gramsPerDV = -1;
    	
    	/**
    	 * 2000 for calories
    	 */
    	public final static int CaloriesTotal = 2000;
    	
    	/**
    	 * Description of nutrient; manually set
    	 */
    	public String description;
    	
    	/**
    	 * This constructor should only be used when the unit is not grams.
    	 * See instance var description.
    	 */
    	public NutrientEntry (NutriUnit unit) {
    		Utils.assertTrue(unit != NutriUnit.GRAMS);
    		this.unit = unit;
    	}
    	
    	/**
    	 * This constructor should only be used when the unit is grams
    	 */
    	public NutrientEntry (NutriUnit unit, int gramsPerDV) {
    		Utils.assertTrue(unit == NutriUnit.GRAMS);
    		this.unit = unit;
    		this.gramsPerDV = gramsPerDV;
    	}
    	
    	public String convert(int amount) {
    		Log.d("NutriTrakDdAdapter", "NutrientEntry.convert()");
    		switch(unit){
    		case GRAMS:
    			if (gramsPerDV == -1){
    				return String.format("%dg", amount);
    			}
    			return String.format("%d%%", (amount * 100) / gramsPerDV);
    		case SELF:
    			return String.format("%d", amount);
    		case DAILY_VALUE:
    			return String.format("%d%%", amount);
    		}
    		return "";
    	}
    }

    /**
     * This is somewhat arbitrary and according to the way nutrition really works
     * and so it will have to be either done by hand or extracted based on a 
     * consistent set of rules from say a website
     */
    public void populateConversionTable() {
    	populateConversionTableWithSampleEntries();
    }
    
    /**
     * Just a temporary solution for the table utilizing manual fill ins
     */
    private void populateConversionTableWithSampleEntries() {
    	// Based on this guide: http://www.netrition.com/rdi_page.html
    	this.nutrientEntries.clear();
    	NutrientEntry calories = new NutrientEntry(NutriUnit.SELF);
    	NutrientEntry fat = new NutrientEntry(NutriUnit.GRAMS, 65);
    	NutrientEntry sugar = new NutrientEntry(NutriUnit.GRAMS, -1);
    	NutrientEntry carbohydrates = new NutrientEntry(NutriUnit.GRAMS, 300);
    	NutrientEntry protein = new NutrientEntry(NutriUnit.GRAMS, 50);
    	NutrientEntry vitaminA = new NutrientEntry(NutriUnit.DAILY_VALUE);
    	NutrientEntry vitaminC = new NutrientEntry(NutriUnit.DAILY_VALUE);
    	this.nutrientEntries.put("calories", calories);
    	this.nutrientEntries.put("fat", fat);
    	this.nutrientEntries.put("sugar", sugar);
    	this.nutrientEntries.put("carbohydrates", carbohydrates);
    	this.nutrientEntries.put("protein", protein);
    	this.nutrientEntries.put("vitaminA", vitaminA);
    	this.nutrientEntries.put("vitaminC", vitaminC);
    	// Wikipedia extract, summarized and edited
    	calories.description = "The Calorie is the unit for digestively available food energy.";
    	fat.description = "Fats are necessary in a regular diet but too many can lead to heart disease.";
    	sugar.description = "Excessive sugar has been correlated with diabetes, obesity, and tooth decay.";
    	carbohydrates.description = "Carbohydrates are not technically essential but are a common source of energy.";
    	protein.description = "Proteins can be used to build muscle and are a key source of nitrogen.";
    	vitaminA.description = "Vitamin A is necessary for maintaining proper vision.";
    	vitaminC.description = "Vitamin C is required for metabolic functions.";
    	Log.d("NutriTrakDdAdapter", "Populated conversion table with sample entries");
    }
    
    public enum ExerciseType {DISTANCE, REPETITIONS};
    public enum ExerciseUnit {MILES, YARDS, REPETITIONS};
    public static class ExerciseData {
    	// what type of exercise is it
    	ExerciseType type;
    	
    	// what units does the exercise use
    	ExerciseUnit unit;
    	
    	// calorie equivalent to one of the exercise unit;
    	double calEquiv;
    	
    	public ExerciseData(ExerciseType type, ExerciseUnit unit, double calEquiv) {
    		this.type = type;
    		this.unit = unit;
    		this.calEquiv = calEquiv;
    	}
    }
    public void populateExerciseTable() {
    	populateExerciseTableWithSampleEntries();
    }
    
    //calEquiv estimated from: http://www.nutristrategy.com/activitylist4.htm using weight of 155lbs
    private void populateExerciseTableWithSampleEntries() {
    	this.exerciseEntries.clear();
    	ExerciseData running = new ExerciseData(ExerciseType.DISTANCE, ExerciseUnit.MILES, 115);
    	ExerciseData swimming = new ExerciseData(ExerciseType.DISTANCE, ExerciseUnit.YARDS, 0.13);		// Avg swim speed: 3mph
    	ExerciseData bicycling = new ExerciseData(ExerciseType.DISTANCE, ExerciseUnit.MILES, 45);
    	ExerciseData elliptical = new ExerciseData(ExerciseType.DISTANCE, ExerciseUnit.MILES, 40);
    	ExerciseData situps = new ExerciseData(ExerciseType.REPETITIONS, ExerciseUnit.REPETITIONS, 3);
    	this.exerciseEntries.put("Running", running);
    	this.exerciseEntries.put("Swimming", swimming);
    	this.exerciseEntries.put("Bicycling", bicycling);
    	this.exerciseEntries.put("Elliptical", elliptical);
    	this.exerciseEntries.put("Situps", situps);
    	Log.d("NutriTrakDdAdapter", "Populated exercise data table with samples");
    }
    
    /**
     * A method for retrieving the ids of the top level food categories
     * 
     * @return	A LinkedList of Strings corresponding to top level category names
     */
    public int[] getTopLevelCategories() {
    	String queryString = 
    		"Select C._id, C.name " +
    		"From categories C " +
    		"Where C.category_id = 0 " + 
    		"Order By C.name ASC";
    	Cursor cur = this.mDb.rawQuery(queryString, null);
    	
    	int[] ids = new int[cur.count()];
    	LinkedList<Integer> retVal = new LinkedList<Integer>();
    	int i = 0;
    	while (cur.next()) {
    		ids[i++] = cur.getInt(0);
    	}
    	
    	return ids;
    }
    
    public String getCategoryName(int id) {
    	String queryString = 
    		"Select C.name " +
    		"From categories C " +
    		"Where C._id = "+id;
    	Cursor cur = this.mDb.rawQuery(queryString, null);
    	
    	cur.first();
    	return cur.getString(0);
    }
    
    /**
     * 
     * A method for retrieving the children of a food category.
     * The children retrieved will be categories, not food, if there are any.
     * To retrieve food children, see the next method under this one
     * 
     * The method this.getCorrespondingIDs() can be used to fetch the IDs of the children 
     * immediately after this method has been invoked
     * 
     * @return	A LinkedList of Strings corresponding to the children of the 
     * 			given category's name.  The returned linked list can be of 0 length if 
     * 			non-leaf but no children.
     */
    public LinkedList<String> getCategoryChildrenOfCategory(String category) {
    	// "Get all categories whose parent's name is category"
    	this.correspondingIDs.clear();
    	
    	String queryString = 
    		"Select C._id " +
    		"From categories C " +
    		"Where C.name = " + "'" + category + "' " +
    		"Limit 1";
     	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	long parentID = sqlStmt.simpleQueryForLong();
    	
    	queryString = 
    		"Select C._id, C.name " +
    		"From categories C " +
    		"Where C.category_id = " + parentID + " " +
    		"Order By C.name ASC";
     	Cursor cur = this.mDb.rawQuery(queryString, null);
    	
    	LinkedList<String> retVal = new LinkedList<String>();
    	while (cur.next()) {
    		int id = cur.getInt(0);
    		String categoryName = cur.getString(1);
    		retVal.add(categoryName);
    		this.correspondingIDs.add(id);
    	}
    	
    	return retVal;
    }
    
    public Cursor getCategoryChildrenOfCategory(long parentID) {

    	return mDb.query(TABLE_CATEGORIES, new String[] {KEY_ROWID, KEY_NAME},
    			KEY_CATEGORY_ID+"="+parentID, null, null, null,
    			KEY_NAME+" ASC");
    }
    
    /**
     * 
     * A method for retrieving the children of a food category.
     * The children retrieved will be food, not categories, if there are any.
     * 
     * The method this.getCorrespondingIDs() can be used to fetch the IDs of the children 
     * immediately after this method has been invoked
     * 
     * @return	A LinkedList of Strings corresponding to the children of the 
     * 			given category's name.  The returned linked list can be of 0 length if 
     * 			non-leaf but no children.
     */
    public LinkedList<String> getFoodChildrenOfCategory(String category) {
    	// "Get all foods whose parent's name is category"
    	this.correspondingIDs.clear();
    	
    	String queryString = 
    		"Select C._id " +
    		"From categories C " +
    		"Where C.name = " + "'" + category + "' " +
    		"Limit 1";
     	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	long parentID = sqlStmt.simpleQueryForLong();
    	
    	queryString = 
    		"Select F._id, F.name " +
    		"From foods F " +
    		"Where F.category_id = " + parentID + " " +
    		"Order By F.name ASC";
     	Cursor cur = this.mDb.rawQuery(queryString, null);
    	
    	LinkedList<String> retVal = new LinkedList<String>();
    	while (cur.next()) {
    		int id = cur.getInt(0);
    		String foodName = cur.getString(1);
    		retVal.add(foodName);
    		this.correspondingIDs.add(id);
    	}
    	
    	return retVal;
    }
    
    public Cursor getFoodChildrenOfCategory(long parentID) {

    	return mDb.query(TABLE_FOOD, new String[] {KEY_ROWID, KEY_NAME},
    			KEY_CATEGORY_ID+"="+parentID, null, null, null,
    			KEY_NAME+" ASC");
    }
    
    /**
     * @param name	The name of the entry that we are searching to make sure is food
     * @return		A boolean value True if the name corresponds to a food or False 
     * 				otherwise (a category, or nonexistent)
     */
    public boolean isFood(String name) {
    	// The LIMIT is so that the db knows not to search further (aka optimization)
    	// There should never actually be more than 1 match
    	String queryString = 
    		"Select Count(*) " +
    		"From foods F " +
    		"Where F.name = " + "'" + name + "' " +
    		"Limit 1";
    	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	long count = sqlStmt.simpleQueryForLong();
    	Utils.assertFalse(count > 1);
    	return count == 1;
    }
    
    /**
     * The returned Cursor will traverse over 2 columns: _id, food name
     * Could be useful for an autocomplete adapter
     * 
     * A Cursor is just like an iterator, except random access is provided and
     * the contents support updates.  See this.getChildren() for sample usage
     * 
     * @return	A Cursor over the entire set of foods in the database
     */
    public Cursor getFoods() {
    	String queryString = 
    		"Select F._id, F.name " +
    		"From foods F " +
    		"Order By F.name ASC";
    	return this.mDb.rawQuery(queryString, null);
    }
    
    /**
     * The returned Cursor will traverse over 2 columns: _id, food name
     * Could be useful for an autocomplete adapter
     * 
     * A Cursor is just like an iterator, except random access is provided and
     * the contents support updates.  See this.getChildren() for sample usage
     * 
     * @param constraint	A sequence of characters that the strings returned must start with
     * @return				A Cursor over the entire set of foods in the database
     */
    public Cursor getFoods(CharSequence constraint) {
    	// See http://www.1keydata.com/sql/sqllike.html
    	if (constraint == null)
    		return this.getFoods();
    	String queryString = 
    		"Select F._id, F.name " +
    		"From foods F " +
    		"Where F.name LIKE " + "'" + constraint + "%' " +
    		"Order By F.name ASC";
    	return this.mDb.rawQuery(queryString, null);
    }
    
    /**
     * @return	A Vector of IDs corresponding to the results of the most recent database
     * 			query that returned an explicit collection (eg a list) of values.
     * 
     * 			See this.getCategoryChildren() etc for reference
     */
    public Vector<Integer> getCorrespondingIDs() {
    	return (Vector<Integer>) this.correspondingIDs.clone();
    }
    
    /**
     * Conversion method from food IDs to names
     * @param foodID	The ID of a food from the database
     * @return			The name of that food
     */
    public String fromFoodIDtoName(long foodID) {
    	String queryString = 
    		"Select F.name " +
    		"From foods F " +
    		"Where F._id = " + foodID + " " +
    		"Limit 1";
    	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	String foodName = sqlStmt.simpleQueryForString();
    	return foodName;
    }
    
    /**
     * Conversion method from food names to food IDs
     * @param foodName	The name of a food in the database
     * @return			The ID corresponding to that food
     */
    public int fromFoodNameToID(String foodName) {
    	String queryString = 
    		"Select F._id " +
    		"From foods F " +
    		"Where F.name = " + "'" + foodName + "' " +
    		"Limit 1";
    	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	int foodID = (int) sqlStmt.simpleQueryForLong();
    	return foodID;
    }
    
    /**
     * Provides a way to fetch meals consumed on a certain day
     * 
     * Note: Due to Date and Time methods being deprecated, it is recommended that
     * the Gregorian Calendar class be used.
     * 
     * See http://code.google.com/android/reference/java/sql/Date.html
     * See http://code.google.com/android/reference/java/util/GregorianCalendar.html
     * 
     * @param date	A calendar object holding time information corresponding to a certain date
     * @return		An array of meal IDs corresponding to meals consumed that day
     */
    public int[] getConsumedMealsOnDate(GregorianCalendar cal) {
    	String queryString =
    		"Select M._id " +
    		"From meals M " +
    		"Where date(M.tstamp) = " + "'" +
    				DateFormat.format("yyyy/MM/dd", cal).toString().replace('/', '-') + "'";
    	
     	Cursor cur = this.mDb.rawQuery(queryString, null);
    	int size = cur.count();
     	int[] retVal = new int[size];
    	
     	int i = 0;
    	while (cur.next()) {
    		int mealID = cur.getInt(0);
    		retVal[i] = mealID;
    		i++;
    	}
    	
    	return retVal;
    }
    
    public int[] getExercisesOnDate(GregorianCalendar cal) {
    	String queryString =
    		"Select M._id " +
    		"From exercises M " +
    		"Where date(M.tstamp) = " + "'" +
    				DateFormat.format("yyyy/MM/dd", cal).toString().replace('/', '-') + "'";
    	
     	Cursor cur = this.mDb.rawQuery(queryString, null);
    	int size = cur.count();
     	int[] retVal = new int[size];
    	
     	int i = 0;
    	while (cur.next()) {
    		int mealID = cur.getInt(0);
    		retVal[i] = mealID;
    		i++;
    	}
    	
    	return retVal;
    }
    
    public Cursor getConsumedMealsOnDateCursor(GregorianCalendar cal) {
    	/*String queryString =
    		"SELECT (_id, tstamp)" +
    		"FROM meals " +
    		"WHERE date(tstamp) = " + "'" +
    				DateFormat.format("yyyy/MM/dd", cal).toString().replace('/', '-') + "'";*/
    	

    	return mDb.query(TABLE_MEALS, new String[] {KEY_ROWID, KEY_TIMESTAMP},
    			"date(tstamp)='"
    			+DateFormat.format("yyyy/MM/dd", cal).toString().replace('/', '-') + "'", 
    			null, null, null, null);
    	
    	//return this.mDb.rawQuery(queryString, null);
    }
    
    /**
     * 
     * Note: This is a converter function.  Not a query function.
     * 
     * @param total					The amount of grams or DV the user is at for a nutrient
     * @param nutrient				A NutrientEntry representing that nutrient
     * @param yourContext			The context that wants to perform the conversion
     * @return						The % the user is at for that nutrient
     */
    public double convertToPercent(int total, NutrientEntry nutrient, Context yourContext) {
		boolean hasPref = false;
		int prefValue = -1;
		SharedPreferences prefs = 
			yourContext.getSharedPreferences(PreferencesView.preferencesFileName, Context.MODE_PRIVATE);
		Map thresholdPrefs = prefs.getAll();
		if (thresholdPrefs.containsKey(nutrient)) {
			hasPref = true;
			prefValue = (Integer) thresholdPrefs.get(nutrient);
		}
		
		//calculate percentage
		float nutrientPercent = 0;
		if (nutrient != null) {
			if (nutrient.unit == NutriTrakDdAdapter.NutriUnit.GRAMS) {
				if (nutrient.gramsPerDV == -1) {
					// arbitrary
					nutrientPercent = (float) total / (hasPref ? prefValue : 100);
				} else {
					nutrientPercent = (float) total / (hasPref ? prefValue : nutrient.gramsPerDV);
				}
			} else if (nutrient.unit == NutriTrakDdAdapter.NutriUnit.SELF) {
				//assume calories
				nutrientPercent = (float) total / 
				(hasPref ? prefValue : NutriTrakDdAdapter.NutrientEntry.CaloriesTotal);
			} else {	// nutrientEntry.unit == DV
				nutrientPercent = (float) total / (hasPref ? prefValue : 100);
			}
		}

		double progressVal = ((int)(nutrientPercent*1000))/10.0;
		return progressVal;
    }
    
    /**
     * Looks the food up and then returns a hashtable whose keys are the nutrient names
     * eg calories, fat, etc and whose values are the respective amounts
     * 
     * Sample usage:
     * getStatsOf("blueberry pie").get("protein")
     * 
     * The stats received are for 1 ounce.
     * So how many grams of protein for 1 ounce.
     * 
     * @param foodID	A food _id whose stats you want to look up 
     * @return			A hashtable representing those stats
     */
    public Hashtable getStatsOfFood(int foodID) {
    	// First, find the stats ID corresponding to this food
    	String queryString = 
    		"Select F.stats_id " +
    		"From foods F " +
    		"Where F._id = " + foodID + " " +
    		"Limit 1";
    	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	int statsID = (int) sqlStmt.simpleQueryForLong();
 
    	// Now generate a query that can get all the nutri columns from the stats row
    	queryString = generateStatQueryString(statsID);
    	Cursor cur = this.mDb.rawQuery(queryString, null);
    	cur.next();
    	
    	// Fill in the hashtable
    	Hashtable retVal = new Hashtable();
    	for (int i = 0; i < STATS_TABLE_NUTRI_KEYS.length; i++) {
    		String nutrient = STATS_TABLE_NUTRI_KEYS[i];
    		retVal.put(nutrient, cur.getInt(i));
    	}
    	
    	return retVal;
    }
    
    /**
     * See comments for other overloaded method in NutriTrakDdAadapter.java
     */
    public Hashtable getStatsOfFood(String foodName) {
    	return this.getStatsOfFood(this.fromFoodNameToID(foodName));
    }
    
    /**
     * Generates a query string that can be used to query the stats table
     * for the stats of a particular food
     * 
     * @param statsID	The stats row we are interested in
     * @return			A string representing an SQL query that can be used to retrieve the stats
     */
    private String generateStatQueryString(int statsID) {
    	/*
    	 * Sample output:
    	 * Select calories, fat, sugar
    	 * From stats 
    	 * Where _id = <statsID>
    	 */
    	StringBuilder q = new StringBuilder();
    	q.append("Select ");
    	
    	for (String nutrient : STATS_TABLE_NUTRI_KEYS) {
    		q.append(nutrient);
    		// If not last element, append a comma
    		if (!nutrient.equals(STATS_TABLE_NUTRI_KEYS[STATS_TABLE_NUTRI_KEYS.length-1]))
    			q.append(", ");
    		else
    			q.append(" ");
    	}
    	
    	q.append("From stats ");
    	q.append("Where _id = " + statsID + " ");
    	q.append("Limit 1");
    	
    	// Log.d("NutriTrackDdAdapter", q.toString());
    	return q.toString();
    }
    
    /**
     * A method for retrieving the foods and nutritional values of a particular meal
     * 
     * @param mealID	The meal we are interested in
     * @return			An instance of a MealStruct representing that meal
     */
    public MealStruct getMealStruct(int mealID) {
    	String queryString =
    		"Select C.food_id, C.quantity, F.name " +
    		"From consumed C, foods F " +
    		"Where C.meal_id = " + mealID + " AND " +
    		"F._id = C.food_id"; 
     	Cursor cur = this.mDb.rawQuery(queryString, null);
     	int size = cur.count();
     	MealStruct retVal = new MealStruct(size);
     	int i = 0;
     	while (cur.next()) {
     		int foodID = cur.getInt(0);
     		int quantity = cur.getInt(1);
     		String foodName = cur.getString(2);
     		retVal.set(i, foodID, foodName, quantity);
     		i++;
     	}
    	return retVal;
    }
    
    public static class MealStruct {
    	int[] foodIDs;
    	String[] foodNames;
    	int[] quantities;
    	
    	public MealStruct (int size) {
    		this.foodIDs = new int[size];
    		this.foodNames = new String[size];
    		this.quantities = new int[size];
    	}
    	
    	// Note: the quantities are in ounces!!!
    	public void set(int index, int foodID, String foodName, int quantity) {
    		this.foodIDs[index] = foodID;
    		this.foodNames[index] = foodName;
    		this.quantities[index] = quantity;
    	}
    }
    
    /**
     * 
     * @param specificDay	A GregorianCalendar object representing a specific day that the
     * 						user ate on, and the day whose meals' nutritional values we 
     * 						want to sum up
     * @return				The total nutritional value of the meals the user ate on that 
     * 						day.  The hashtable's keys are nutrients, eg calories, whereas
     * 						the values are integers representing the amount of grams or
     * 						DV the user is at, depending on the natural unit imposed
     * 						(see the conversion table present at this.nutrientEntries)
     */
    public Hashtable<String, Integer> getTotalStatsOfDay(GregorianCalendar specificDay) {
    	/*
    	 * Initialize total nutritional hashtable
    	 */
    	Hashtable<String,Integer> total = new Hashtable<String,Integer>();
    	for (String nutrient : STATS_TABLE_NUTRI_KEYS) {
    		total.put(nutrient, 0);
    	}
    	/*
    	 * Fetch all the meals consumed on a specific day
    	 */
    	int[] mealIDs = this.getConsumedMealsOnDate(specificDay);
    	/*
    	 * For each meal consumed
    	 * 		For each type of food consumed in that meal
    	 * 			For each type of nutrient in that food
    	 * 				Get the contribution of nutrition (quantity*food's stats)
    	 * 				Add to a running total of nutritional values
    	 */
    	for (int i = 0; i < mealIDs.length; i++) {
    		Hashtable mealStats = this.getStatsOfMeal(mealIDs[i]);
			Enumeration e = mealStats.keys();
			while (e.hasMoreElements()) {
				String nutrient = (String) e.nextElement();
				int runningTotal = total.get(nutrient);
				// Add new contribution to preexisting total
				total.put(nutrient, runningTotal + (Integer) mealStats.get(nutrient));
			}
    	}
    	return total;
    }
    
    public Hashtable<String, Integer> getTotalStatsOfDayForNutrient(GregorianCalendar specificDay, String nutrientName) {
    	Hashtable<String,Integer> total = new Hashtable<String,Integer>();
    	total.put(nutrientName, 0);    	
    	int[] mealIDs = this.getConsumedMealsOnDate(specificDay);
    	for (int i = 0; i < mealIDs.length; i++) {
    		Hashtable mealStats = this.getStatsOfMeal(mealIDs[i]);
			Enumeration e = mealStats.keys();
			while (e.hasMoreElements()) {
				String nutrient = (String) e.nextElement();
				if (!nutrient.equals(nutrientName)) {
					continue;
				}
				int runningTotal = total.get(nutrient);
				// Add new contribution to preexisting total
				total.put(nutrient, runningTotal + (Integer) mealStats.get(nutrient));
			}
    	}
    	return total;
    }
  
    /**
     * get stats of period -- just returns random integers for now
     * 
     * @deprecated No longer used
     * @param GregorianCalendar start 		start of period
     * @param String			nutrient	nutrient to track
     * @param in				num_days	number of days in period
     * @returns int[] with values from 0-100 representing totals for those days
     */
    
    public int[] getStatsOfPeriod(GregorianCalendar start, String nutrient, int num_days){
    	Random r = new Random();
    	
    	int ret[] = new int[num_days];
    	
    	for (int i = 0; i < num_days; i++){
    		ret[i] = Math.abs(r.nextInt()) % 100;
    	}
    	
    	return ret;
    }
    

    /*
     * For the week that day is in,
     * Get stats of each day for a nutrient, return in array.
     * 
     * For each day:
     * 		Get stats of day for particular nutrient
     * 		
     * 		
     * 			
     */
    public int[] getStatsOfPeriod(GregorianCalendar cal, String nutrient, Context yourContext){
    	cal = Utils.cloneGregorianCalendar(cal);
    	while (cal.get(GregorianCalendar.DAY_OF_WEEK) != GregorianCalendar.SUNDAY) {
    		cal.add(GregorianCalendar.DAY_OF_WEEK, -1);
    	}
    	Log.d("NutriTrakDdAdapter", DateFormat.format("yyyy/MM/dd", cal).toString().replace('/', '-'));
    	
    	int ret[] = new int[7];
    	
    	for (int i = 0; i < 7; i++){
    		Hashtable<String, Integer> total = this.getTotalStatsOfDayForNutrient(cal, nutrient);
    		ret[i] = (int) this.convertToPercent(total.get(nutrient), this.nutrientEntries.get(nutrient), yourContext);
    		cal.add(GregorianCalendar.DAY_OF_MONTH, 1);
    	}
    	
    	return ret;
    }
    
    /**
     * Get the stats of a meal in the form of a hashtable.
     * See getTotalStatsOfDay for sample usage
     * 
     * @param mealID	mealID of that meal	
     * @return			a hashtable of nutrient strings to values
     */
    public Hashtable<String, Integer> getStatsOfMeal(int mealID) {
    	/*
    	 * Initialize total nutritional hashtable
    	 */
    	Hashtable<String,Integer> total = new Hashtable<String,Integer>();
    	for (String nutrient : STATS_TABLE_NUTRI_KEYS) {
    		total.put(nutrient, 0);
    	}
    	/*
    	 * Retrieve the appropriate meal struct
    	 */
    	MealStruct meal = this.getMealStruct(mealID);
    	/*
    	 * Iterate over foods in meal, get nutritional value
    	 */
		for (int i = 0; i < meal.foodIDs.length; i++) {
			int foodID = meal.foodIDs[i];
			int quantity = meal.quantities[i];
			
			Hashtable foodStats = this.getStatsOfFood(foodID);
			Enumeration e = foodStats.keys();
			while (e.hasMoreElements()) {
				String nutrient = (String) e.nextElement();
				int runningTotal = total.get(nutrient);
				// Add new contribution to preexisting total
				total.put(nutrient, runningTotal + (quantity * (Integer) foodStats.get(nutrient)));
			}
		}
		return total;
    }
    
    /*
     * You're going to have to do some minor parsing (probably with a builtin
     * java library) to get the stuff you want out of the timestamp.
     * or maybe there's a constructor or something that will do it for you,
     * look around I suppose.
     */
    public String getTimeOfConsumption(int mealID) {
    	String queryString =
    		"Select M.tstamp " +
    		"From meals M " +
    		"Where M._id = " + mealID + " " + 
    		"Limit 1";
    	
    	Cursor cur = this.mDb.rawQuery(queryString, null);
    	cur.first();
    	String tStamp = cur.getString(0);
    	return tStamp;
    }
    
    public String getTimeExercised(int id) {
    	String queryString =
    		"Select M.tstamp " +
    		"From exercises M " +
    		"Where M._id = " + id + " " + 
    		"Limit 1";
    	
    	Cursor cur = this.mDb.rawQuery(queryString, null);
    	cur.first();
    	String tStamp = cur.getString(0);
    	return tStamp;
    }
    
    public Cursor getExerciseInfo(int id) {
    	String queryString = 
    		"Select * " +
    		"From exercises "+
    		"Where _id="+id;
    	
    	return this.mDb.query(TABLE_EXERCISES, new String[] {KEY_TIMESTAMP, KEY_DURATION, KEY_NAME}, 
    			KEY_ROWID+"="+id, null, null, null, null);
    }
    
    // Note: DEPRECATED!!!
    /**
     * Retrieves a string determining the unit of the food, eg serving, floz, etc
     * @param foodIDID	The ID of the food we're interested in
     * @return			A String representing the unit type
     * @deprecated
     */
    public String getFoodUnits(int foodID) {
    	String queryString = 
    		"Select F.units " + 
    		"From foods F " + 
    		"Where F._foodID = " + foodID + " " +
    		"Limit 1";
    	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	String unit = sqlStmt.simpleQueryForString();
    	return unit;
    }

    /**
     * Create a new meal using the specified arguments.
     * 
     * @param 	linked list of food id's that were eaten
     * @param 	linked list of quantities that were eaten matching the food array
     * 			Note: the quantities are in ounces!!
     * @return	the ID of the meal created
     */
    public long createMeal(LinkedList<Integer> foods, LinkedList<Integer> quantities)
    {
    	Utils.assertTrue(foods!= null && quantities!= null);
    	Utils.assertTrue(foods.size() == quantities.size());
    	
    	// (Agreed upon during meeting to assume time of entry = time of meal)
    	// (Can use a TimePickerDialog in UI later if change mind)
    	Timestamp timeStamp = Utils.getTimeStamp();	
    	
    	// Create the row to be added to the meal table
    	ContentValues mealRow = new ContentValues();
    	mealRow.put(KEY_TIMESTAMP, timeStamp.toString());
    	
    	// ID of the row from the meals table 
    	long mealID = this.mDb.insert("meals", null, mealRow);
    	Utils.assertTrue(mealID != -1);

    	for (int i = 0; i < foods.size(); i++) {
    		increaseFavoritism(foods.get(i));
    		ContentValues consumedRow = new ContentValues();
    		consumedRow.put(KEY_FOOD_ID, foods.get(i));
    		consumedRow.put(KEY_QUANTITY, quantities.get(i));
    		consumedRow.put(KEY_MEAL_ID, mealID);
    		long consumeRowID = this.mDb.insert("consumed", null, consumedRow);
    		Utils.assertTrue(consumeRowID != -1);
    	}
    	return mealID;
    }
    
    
    
    /**
     * Find old favoritism value of a food, bump by 1
     * 
     * Note: This hasn't been tested 
     */
    public int increaseFavoritism(int foodID) {
    	String queryString = 
    		"Select F.favoritism " + 
    		"From foods F " + 
    		"Where F._id = " + foodID + " " +
    		"Limit 1";
    	SQLiteStatement sqlStmt = this.mDb.compileStatement(queryString);
    	int oldFavoritism = (int) sqlStmt.simpleQueryForLong();
    	ContentValues cv = new ContentValues();
    	cv.put(KEY_FAVORITISM, oldFavoritism+1);
    	Utils.assertTrue(this.mDb.update(TABLE_FOOD, cv, KEY_ROWID+"="+foodID, null) == 1);
    	return oldFavoritism+1;
    }
    /**
     * 
     * @param exerciseNames		A linked list of strings representing the exercise names
     * @param durations			A linked list of ints representing the durations
     * @return					An array of longs containing the exercise IDs
     */
    public long[] createExercises(LinkedList<String> exerciseNames, LinkedList<Integer> durations) {
    	Utils.assertTrue(exerciseNames.size() == durations.size());
    	Timestamp timeStamp = Utils.getTimeStamp();
    	long[] exerciseIDs = new long[exerciseNames.size()];
    	for (int i = 0 ; i < exerciseNames.size(); i++) {
    		ContentValues cv = new ContentValues();
    		cv.put(KEY_DURATION, durations.get(i));
    		cv.put(KEY_NAME, exerciseNames.get(i));
    		cv.put(KEY_TIMESTAMP, timeStamp.toString());
    		long exerciseRowID = this.mDb.insert(TABLE_EXERCISES, null, cv);
    		Utils.assertTrue(exerciseRowID != -1);
    		exerciseIDs[i] = exerciseRowID;
    	}
    	return exerciseIDs;
    }
    
    /**
     * 
     * @param limit	The number of foods to return
     * @return		A cursor of the most favorite foods, with the limit applied (may be less)
     * 
     * Note: This hasn't been tested 
     */
    public Cursor getFavoriteFoods(int limit) {
    	String queryString = 
    		"Select F._id, F.name " + 
    		"From foods F " + 
    		"Where F.favoritism > 0 " + 
    		"Order By F.favoritism DESC " + 
    		"Limit " + limit;
    	return this.mDb.rawQuery(queryString, null);
    }
    
    /**
     * Update a meal that is already in the database
     * 
     * @param meal_id
     * @param foods
     * @param quantities
     */
	public void updateMeal(long meal_id, LinkedList<Integer> foods,
		LinkedList<Integer> quantities) {
    	
		Utils.assertTrue(foods != null && quantities != null);
    	Utils.assertTrue(foods.size() == quantities.size());
    	Utils.assertTrue(meal_id != -1);
    	
    	//TODO: adjust time stamp first??
    	
    	// since the number of items could have changed theoretically, we delete the
    	// existing records
    	String queryString = String.format("%s = %d",
    		KEY_MEAL_ID, meal_id);
    	mDb.delete("consumed", queryString, null);
    	
    	// now we just insert new items into the table as if it were a newly-created meal
    	for (int i = 0; i < foods.size(); i++) {
    		increaseFavoritism(foods.get(i));
    		ContentValues consumedRow = new ContentValues();
    		consumedRow.put(KEY_FOOD_ID, foods.get(i));
    		consumedRow.put(KEY_QUANTITY, quantities.get(i));
    		consumedRow.put(KEY_MEAL_ID, meal_id);
    		long consumeRowID = this.mDb.insert("consumed", null, consumedRow);
    		Utils.assertTrue(consumeRowID != -1);
    	}
	}
}
